@@ -4,6 +4,7 @@ import json
 import math
 import struct
 import time
+from abc import ABC
 from collections.abc import Callable, Coroutine
 from typing import Any, TypeVar, final
 
@@ -71,6 +72,7 @@ WASH_N_FILL_DOCK = [
     RoborockDockTypeCode.p10_pro_dock,
     RoborockDockTypeCode.s8_maxv_ultra_dock,
     RoborockDockTypeCode.qrevo_s_dock,
+    RoborockDockTypeCode.qrevo_curv_dock,
 ]
 RT = TypeVar("RT", bound=RoborockBase)
 EVICT_TIME = 60
@@ -141,19 +143,22 @@ class ListenerModel:
     cache: dict[CacheableAttribute, AttributeCache]
 
 
-class RoborockClientV1(RoborockClient):
+class RoborockClientV1(RoborockClient, ABC):
+    """Roborock client base class for version 1 devices."""
+
     _listeners: dict[str, ListenerModel] = {}
 
-    def __init__(self, device_info: DeviceData, logger, endpoint: str):
-        super().__init__(endpoint, device_info)
+    def __init__(self, device_info: DeviceData, endpoint: str):
+        """Initializes the Roborock client."""
+        super().__init__(device_info)
         self._status_type: type[Status] = ModelStatus.get(device_info.model, S7MaxVStatus)
-        self._logger = logger
         self.cache: dict[CacheableAttribute, AttributeCache] = {
             cacheable_attribute: AttributeCache(attr, self) for cacheable_attribute, attr in get_cache_map().items()
         }
         if device_info.device.duid not in self._listeners:
             self._listeners[device_info.device.duid] = ListenerModel({}, self.cache)
         self.listener_model = self._listeners[device_info.device.duid]
+        self._endpoint = endpoint
 
     def release(self):
         super().release()
@@ -377,20 +382,17 @@ class RoborockClientV1(RoborockClient):
                             if queue and queue.protocol == protocol:
                                 error = data_point_response.get("error")
                                 if error:
-                                    queue.resolve(
-                                        (
-                                            None,
-                                            VacuumError(
-                                                error.get("code"),
-                                                error.get("message"),
-                                            ),
-                                        )
+                                    queue.set_exception(
+                                        VacuumError(
+                                            error.get("code"),
+                                            error.get("message"),
+                                        ),
                                     )
                                 else:
                                     result = data_point_response.get("result")
                                     if isinstance(result, list) and len(result) == 1:
                                         result = result[0]
-                                    queue.resolve((result, None))
+                                    queue.set_result(result)
                             else:
                                 self._logger.debug("Received response for unknown request id %s", request_id)
                         else:
@@ -420,6 +422,12 @@ class RoborockClientV1(RoborockClient):
                                     consumable = Consumable.from_dict(value)
                                     for listener in self.listener_model.protocol_handlers.get(data_protocol, []):
                                         listener(consumable)
+                                else:
+                                    self._logger.warning(
+                                        f"Unknown data protocol {data_point_number}, please create an "
+                                        f"issue on the python-roborock repository"
+                                    )
+                                    self._logger.info(data)
                                 return
                             except ValueError:
                                 self._logger.warning(
@@ -444,13 +452,13 @@ class RoborockClientV1(RoborockClient):
                         if queue:
                             if isinstance(decompressed, list):
                                 decompressed = decompressed[0]
-                            queue.resolve((decompressed, None))
+                            queue.set_result(decompressed)
                         else:
                             self._logger.debug("Received response for unknown request id %s", request_id)
                 else:
                     queue = self._waiting_queue.get(data.seq)
                     if queue:
-                        queue.resolve((data.payload, None))
+                        queue.set_result(data.payload)
                     else:
                         self._logger.debug("Received response for unknown request id %s", data.seq)
         except Exception as ex:
